
/*This program gives tests on a large library of published juvenile 
books.  It also maintains a printable record of each student's performance
on the tests.  

                  GNU ECLECTIC READER  version 3/31/06 
                  
Copyright (C) 2006 by John Slayton

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warrangty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Boston, MA, 02110-1301, USA.

Author's address as of 9/17/05 is 260 Sundance Ct., Azusa, CA, 91702.

                             PROGRAM CODING 

     In the following source code, the program methods are arranged in 
alphabetical order, below main.  */

#include <cups/cups.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#ifdef HAVE_LIBCURSES
#include <curses.h>
#else
#include <ncurses/curses.h>
#endif

                           /*  GLOBAL VARIABLES  */

int argc;

/*  As students are added or dropped, this total will change.  */
int Class_size;

/*  This holds the total number of books in the library.  */
int Library_size = 0;

/*  This is a flag to return to top level from embedded routines.  */
int Panic = 0;

/*  This indexing variable will hold the roster number of a student.  */ 
int Stu = 0;

/*  This indexing variable will hold the call number of a book.  */
int Bknr;

char *argv;

/*  Here's a list of 400 books with accompanying tests.  */ 
char Book[400][2000];

/*  Here's a buffer to assemble long strings.  */
char Buf[20000] = "";

/*  This is a list of students, each with an appended test record.  */
char Name[100][500];

/*  This holds the teacher security code.  */
char Tc[4];

FILE *fp;

struct tm *loctime;

time_t *t = NULL;
time_t curtime;

                            /*  FUNCTIONS  */

int add_student(void);
int add_book(void);
int download_books(void);
int download_roster(void);
int drop_student(void);
int drop_book(void); 
int format_for_printer(void);
int format_for_video(void);
int print_help(void);
int print_version(void);

/*  These sections are parts of a template for ps printing.  This program 
inserts text between these sections, then strings them all together for 
printing with a system call to lpr  */

int sectionaa(void);
int sectionbb(void);
int sectioncc(void);
int sectiondd(void);
int sectionee(void);

int select_book(void);
int select_student(void);
int show_books(int);
int show_roster(int);
int student_clearance(void);
int teacher_clearance(void);
int testing(void);
int type_anything(void);
int upload_books(void);
int upload_roster(void);

                        /*  PROGRAM ENTRY POINT  */

int
main (int argc, char **argv)
{
  /*  Vertical print coordinate 'h' will change to scroll text up  */
  int h;
  int e;
  int run_options_only = 0;
  long int a;

  /*  Set Ncurses Parameters  */
  initscr();
  curs_set(0);
  cbreak();
  noecho();
  nonl();
  intrflush (stdscr, FALSE);
  keypad (stdscr, TRUE);
  start_color();
  init_pair (1, COLOR_BLACK, COLOR_WHITE);
  init_pair (2, COLOR_RED, COLOR_WHITE);
  init_pair (3, COLOR_GREEN, COLOR_WHITE);
  init_pair (4, COLOR_BLUE, COLOR_WHITE);
  init_pair (5, COLOR_MAGENTA, COLOR_WHITE);
  bkgd (COLOR_PAIR(1));

         /*  Start by Handling Program Options  */
  int c;
  int digit_optind = 0;

  while (1)
    { int this_option_optind = optind ? optind : 1;
      int option_index = 0;
      static struct option long_options[] =
                                            {
                                              {"version", 0, 0, 0},
                                              {"help", 0, 0, 0},
                                              {0,0,0,0}
                                            };
      c = getopt_long (argc, argv, "", long_options, &option_index);
      if (c == -1) break;

      switch (c)
        {
          case 0:

            if (long_options[option_index].name == "version") print_version();

            if (long_options[option_index].name == "help") print_help();
 
            if (optarg)  printf (" with arg %s", optarg);
            run_options_only = 1;
            break;

          case '2':
            if (digit_optind != 0 && digit_optind != this_option_optind)
            printf ("digits occur in two different argv-elements.\n");
            digit_optind = this_option_optind;
            printf ("option %c\n", c);
            break;

          case '?':
             break;

          default:
             printf ("?? getopt returned character code 0%o ??\n", c);
        }
    }

      if (optind < argc)
        {
          printf ("non-option ARGV-elements: ");
          while (optind < argc) printf ("%s ", argv[optind++]);
          printf ("\n");
        }

      if (run_options_only == 1)
        {
          getch();
          endwin();
          exit (0);
        }

 
  /*  Read student records, library data, and teacher security code.  */
  download_roster();
  download_books();
  fp = fopen ("/usr/local/com/reader/Tc", "r");
  fgets (Tc, 5, fp);
  fclose (fp);

  /*  Present Introduction  */
  start:
  clear();
  attron (COLOR_PAIR(2));
  mvprintw (5, 22, "WELCOME TO GNU ECLECTIC READER\n\n\n\n\n\n");
  attron (COLOR_PAIR(5));
  printw ("                              with apologies to\n"
         "                              WILLIAM  MCGUFFEY\n"
         "                                 1800 - 1873");
    
  /*  Scroll up Copyright Notice  */
  refresh();
  attron (COLOR_PAIR(1));
  for (h = 25;  h > 17; h--) 
    { 
      mvprintw (h, 32, "By John Slayton\n");
      mvprintw (h + 1, 21, "Vers 3/31/06       Copyright (C) 2006\n");
      mvprintw (h + 2, 21, "                                       ");
      refresh();    
      for (a = 0; a < 20000000; a++) ;
    }
  
  /*  Wait a while before changing display  */
  for (a = 0;  a < 60; a++) 
    {
      halfdelay(1);
      if (getch() > 0) break;
    }
  nocbreak();
  cbreak();
  
  /*  Print GPL notice  */
  begin:
  clear();
  mvprintw (10, 35, "DISTRIBUTABLE UNDER");
  mvprintw (11, 32, "GNU GENERAL PUBLIC LICENSE");
  mvprintw (12, 33, "FREE SOFTWARE FOUNDATION");
  mvprintw (13, 37, "51 FRANKLIN ST"); 
  mvprintw (14, 36, "BOSTON, MA, 02110");
  mvprintw (15, 35, "PUSH D FOR DETAILS.");
  attron (COLOR_PAIR(2));
  mvprintw (16, 34, "Then push q to return.");
  mvprintw (22, 2, "   Push ENTER to start.");
  refresh();
   
  /*  Wait some more before toggling display.  */
  for (a = 0;  a < 60; a++) 
    {
      halfdelay(1);
      e = getch();
      if (e > 0) break;
    }
  nocbreak();
  cbreak();
 
  /*  If "D" was pressed, display GPL by system call to cat | less.  */
  if ((e == 67) || (e == 100))
    {
       system("cat /usr/local/share/doc/reader/COPYING | less");
    }
  if (e != 13) goto start;

  /*  Print the program menu. */
  while (1 == 1)
    { 
      curs_set(0);
      clear();
      mvprintw (5, 30, "GNU ECLECTIC READER");      

      attron (COLOR_PAIR(5));
      mvprintw (7, 31,"STUDENT FUNCTIONS");
      mvprintw (9, 25, "A - Take Test");
      mvprintw (10, 25, "B - Read Booklist");
      mvprintw (11, 25, "C - Read Roster");

      attron (COLOR_PAIR(4)); 
      mvprintw (13, 31, "TEACHER FUNCTIONS");
      mvprintw (15, 25, "D - Show Student Record"); 
      mvprintw (16, 25, "E - Print Student Record");
      mvprintw (17, 25, "F - Print Class Profile");
      mvprintw (18, 25, "G - Add Student");
      mvprintw (19, 25, "H - Drop Student");
      mvprintw (20, 25, "I - Add Book to Library");
      mvprintw (21, 25, "J - Drop Book from Library");
      attron (COLOR_PAIR(1));
      mvprintw (23, 25, "K - Help");
      refresh();

      /*  Select function, using either upper or lower case.  */
      a = 0;
      while ((a < 64) || (a > 75))
        {
          a = getch();
          if (a > 96) a = a - 32;
        }
      clear();
      switch (a)
        {
                            /*  TAKE TEST  */
          case 65:
            testing();
            break;
                            /* READ BOOK LIST */
          case 66:
            select_book();    
            break;
                            /*  READ ROSTER  */
          case 67:
            select_student();
            break;
                            /*  SHOW STUDENT RECORD  */
          case 68:
            teacher_clearance(); 
            select_student();
            format_for_video();
            break;
                            /*  PRINT STUDENT RECORD  */
          case 69:
            teacher_clearance();
            if (Panic == 1) break;
            select_student();
            format_for_printer();
            if (Panic ==1) break;
            system("lpr /usr/local/com/reader/report.ps");
            break;
                            /*  PRINT CLASS PROFILE  */
          case 70:
            teacher_clearance();
            if (Panic == 1) break;
            for (Stu = 0; Stu < Class_size; Stu++)
              {
                format_for_printer();
                if (Panic == 1) break;
                system("lpr /usr/local/com/reader/report.ps");
              }
            break;
                            /*  ADD STUDENT  */
          case 71:
            teacher_clearance();
            if (Panic == 1) break;
            add_student();
            if (Panic == 1) break;
            upload_roster();
            break;        
                            /*  DROP STUDENT  */
          case 72:
            teacher_clearance();
            if (Panic == 1) break;
            drop_student();
            break;
                            /*  ADD BOOK  */
          case 73:
            teacher_clearance();
            if (Panic == 1) break;
            add_book();
            break;
                            /* DROP BOOK */
          case 74:
            teacher_clearance();
            if (Panic == 1) break;
            drop_book();
            break;

                           /*  HELP  */
          case 75:
            teacher_clearance();
            if (Panic == 1) break;
            print_help();
            break;
        }
      Panic = 0;
    }
  return 0;
}

/*  This routine adds a new book and test to the library file.  Titles
    are arranged in alphabetical order.  */
int
add_book()
{
  int a;
  int answer_nr;
  int question_nr;
  int print_line;
  int g;
  int n;
  char zo[2];

  /*  Put name of the book into Buf.  */
  attron (COLOR_PAIR(2));
  mvprintw (5, 35, "Name the book.");
  mvprintw (6, 35, "");
  attron (COLOR_PAIR(4));
  strcpy (Buf, "");
  type_anything();
  if (Panic == 1) return 0;
  strcat (Buf, "%");
  mvprintw (8, 35, "%s", Buf);
    
  /*  Add a question.  */
  for (question_nr = 0; question_nr < 25; question_nr++)
    { 
      clear();
      attron (COLOR_PAIR(2));
      mvprintw (2, 25, "Type in question number %c.", question_nr + 49);
      mvprintw (3, 5,"");
      attron (COLOR_PAIR(4));
      type_anything();
      if (Panic == 1) return 0;

      strcat (Buf, "#");
 
      /*  Enter Possible Answers  */
      print_line = 5;
      attron (COLOR_PAIR(2));
      mvprintw (print_line, 25, "Type in four possible answers.");
      attron (COLOR_PAIR(4));
      for (answer_nr = 1; answer_nr < 5; answer_nr++)
        { 
          print_line++;
          mvprintw (print_line, 25, "%c  ", (answer_nr+96));
          if (answer_nr == 1) strcat (Buf, "a._");
          if (answer_nr == 2) strcat (Buf, "b._");
          if (answer_nr == 3) strcat (Buf, "c._");
          if (answer_nr == 4) strcat (Buf, "d._");
          type_anything();
          if (Panic == 1) return 0;

          if (answer_nr < 4) strcat (Buf, "#");
          else strcat (Buf, "%");
        }

      /*  Enter Right Answer  */
      print_line++;
      print_line++;
      attron (COLOR_PAIR(2));
      mvprintw (print_line, 25, "Which is the right answer? (a to d)");
      attron (COLOR_PAIR(4));
      a = 0;
      while (((a < 97) || (a > 100)) && (a != '\033')) a = getch();

      /*  Jump out on ESC.  */
      if (a == '\033') return 0; 

      print_line++;
      mvprintw (print_line, 25, "%c", a);
      zo[0] = (char)(a);
      strncat (Buf, &zo[0], 1);
      strcat (Buf, "%");
      print_line++;
      print_line++;
      attron (COLOR_PAIR(2));
      mvprintw (print_line, 25, "Add another question?  (Y / N)");
      attron (COLOR_PAIR(4));
      a = getch();

      /*  Jump out on ESC.  */
      if (a == '\033') return 0;

      if ((a != 89) && (a != 121)) break;
    }

  /*  Ask for points of book  */
  strcat (Buf, "~");
  attron (COLOR_PAIR(2));
  print_line++;
  print_line++;
  mvprintw (print_line, 25, "How many points is this book worth?");
  print_line++;
  mvprintw (print_line, 25, "(Use 2 digits.  Example: enter 08 for 8)");
  attron (COLOR_PAIR(4));

  /*  Get first digit.  */ 
  a = 0;
  while (((a < 48) || (a > 57)) && (a != '\033')) a = getch();

  /*  Jump out on ESC.  */
  if (a == '\033') return 0;

  print_line++;
  mvprintw (print_line, 25, "%c", a);
  zo[0] = (char)(a);
  strncat (Buf, &zo[0], 1);

  /*  Get second digit.  */
  a = 0;
  while (((a < 48) || (a > 57)) && (a != '\033')) a = getch();

  /*  Jump out on ESC.  */
  if (a == '\033') return 0;

  printw ("%c", a);
  zo[0] = (char)(a);
  strncat (Buf, &zo[0], 1);
    
  /*  Ask for difficulty level of book.  */
  attron (COLOR_PAIR(2));
  print_line++;
  print_line++;
  mvprintw (print_line, 25, "     What level is this book?");
  print_line++;
  mvprintw (print_line, 25, "  (Enter number between 1 and 9.)");
  attron (COLOR_PAIR(4));

  /*  Get a digit between 1 and 9.  */
  a = 0;
  while (((a < 49) || (a > 57)) && (a != '\033')) a = getch();

  /*  Jump out on ESC.  */
  if (a == '\033') return 0;

  zo[0] = (char)(a);
  strncat (Buf, &zo[0], 1);

  /*  Insert the new listing into the library alphabetically.  */
  clear();
  for (g = 0, n = Library_size; (n > 0) && (g < 30); g++) 
    {
      if (Book[n - 1][g] < Buf[g])
        {
          g = 30; 
          break;
        }          
      if (Book[n - 1][g] > Buf[g])
        {
          g = (-1);
          n--;
        }
    }
  for (a = Library_size; a > n; a--) strcpy(Book[a], Book[a - 1]);
  strcpy (Book[n], Buf);
  Library_size++;
  upload_books();
  clear();
  return 0;
}

/*  This routine inserts a new name into the array Name[][], maintaining
    alphabetical order.  */
int
add_student()
{ 
  int a;

  /*  Indexes each character in a file.  */
  int file_offset;

  /*  Numbers a file for each student.  */
  int roster_nr;

  char keystroke[1];

  /*  Temporary file for new student  */
  char newby[80];

  /*  Check to See If Roster is Full  */
  if (Class_size == 100)
    { 
      mvprintw (11, 30, "Class is Full");
      getch();
      return 0;
    } 

  /*  Input Name of New Student  */
  mvprintw (5, 30, "Who's new?");
  mvprintw (7, 7, "");
  strcpy (Buf, "");
  type_anything();  /*  Get name from standard input  */
  if (Panic == 1) return 0;
  strcat (Buf, "%"); /*  Add delimiter  to end name field  */
  strcpy (newby, Buf);  /*  Transfer name to temp buffer  */

  /*  Create Student Password.  */
  mvprintw (10, 30, "Enter Student 4 Digit Password.");
  mvprintw (12, 30, "Use lower case letters only.");
  mvprintw (14, 30, "");

  /*  Return to this point if error is made below.  */
  retry:
  strcpy (Buf, "");
  for (a = 0; a < 4; a++)
    {
      keystroke[0] = getch();

      /*  Jump out on ESC.  */
      if (keystroke[0] == '\033')
        {
          Panic = 1;
          return 0;
        } 

      printw ("*"); 
      if ((keystroke[0] < 91) || (keystroke[0] > 126))
        { 
          mvprintw (16, 30, "Try again, with lower case letters only.");
          mvprintw (18, 30, "         ");
          mvprintw (18, 30, "");
          a = 5;
        }
      else strncat(Buf, keystroke, 1);
    }
  if (a > 4) goto retry;

  /*  Add password to temporary buffer  */
  strcat (newby, Buf);

  /*  Add delimiter  to end password field  */
  strcat (newby, "%");

  /*  Locate Alphabetical Position of New Name  */
  for (file_offset = 0, roster_nr = Class_size;  (roster_nr >
  0) && (file_offset < 30); file_offset++)
    { 
      if (Name[roster_nr - 1][file_offset] < newby[file_offset])
        {
          file_offset = 30;
          break;
        }
      if (Name[roster_nr-1][file_offset]>newby[file_offset])
        {
          file_offset--;
          roster_nr--;
        }
    }
 
  /*  Make Space in Roster & Insert New Name  */
  for (a = Class_size; a > roster_nr - 1; a--) strcpy (Name[a + 1], Name[a]);
  strcpy (Name[roster_nr], newby); /*  Insert new student name  */
  Class_size++;    
  return 0;
}

/*  This routine reads the library holdings from the hard drive, placing 
    them in the array Book[][].  */    
int
download_books()
{
  int a = 0;
  int letter_of_title;
  char buffer1[800000];

  /*  Download the number of books in char form & convert to binary.  */
  fp = fopen("/bugstash/Library_size", "r");
  strcpy (Buf, "");
  fgets (Buf, 4, fp);
  Library_size = atoi (Buf);

  /*  Download all the book titles in one long string.  */
  strcpy (Buf, "/bugstash/bundle");
  fp = fopen (Buf, "r");
  fgets (buffer1, 500000, fp);
  fclose (fp);

  /*  Break the string into individual book files.  */
  for (Bknr = 0; Bknr < Library_size ; Bknr++)
    {
      letter_of_title = 0;
      while ((buffer1[a] != '@') && (buffer1[a] != 0))
        {
          Book[Bknr][letter_of_title] = buffer1[a];
          letter_of_title++;
          a++;
        }
      if (buffer1[a] == 0) return 0;
      if (buffer1[a] == '@') a++;
    }
  return 0;
}

/*  This routine reads the student files (with test records) from the 
hard drive and places them in the array Name[][].  */
int
download_roster()
{
  int a;
  char file_size[5];
  char zo[2];

  /*  Download the class size as ASCII number, then change to int.  */
  fp = fopen ("/bugstash/Class_size", "r");
  strcpy (file_size, ""); /*  Clear buffer  */
  fgets(file_size, 4, fp);
  fclose(fp);
  Class_size = atoi (file_size);

  /*  Open individual student files.  */
  for (a = 0;  a < Class_size; a++)
    {
      strcpy (Buf, "/bugstash/Name");
      zo[0] = (char)((a / 10) + 48);
      strcat (Buf, &zo[0]);
      zo[0] = (char) ((a % 10) + 48);
      strcat (Buf, &zo[0]);
      fp = fopen(Buf, "r");
      fgets (Name[a], 500, fp);
      fclose(fp);
    }
  return 0;
}

/*  Remove a student from the roster.  */
int
drop_student()
{
  mvprintw (2, 5, "Who left?");
  select_student();

  /* All roster names move down 1 line from delete point to roster end.  */ 
  for (Stu = Stu; Stu < Class_size + 1; Stu++) strcpy (Name[Stu], Name[Stu+1]);

  /*  Erase duplicated last name from its old position.  */
  strcpy (Name[Stu], "");

  Class_size--;
  upload_roster();
  return 0;
}

/*  Remove a book file from the library.  */
int
drop_book()
{
  /* Returns book number  */
  select_book();

  if (Panic == 1) return 0;
  for (; Bknr <= Library_size + 1; Bknr++)
    {
      strcpy (Book[Bknr], Book[Bknr+1]);
    }
  Library_size--;
  upload_books();
  return 0;
}

/*  This routine lays out a page for printing and stores it as report.ps.  */ 
int
format_for_printer()
{
  int delimiters = 0;
  int file_offset;

  /*  Vertical coordinate for PS printing (binary)  */
  int line_b = 0;

  int total = 0;
  char zo[4] = "";

  /*  Has the student taken any tests?  */
  for ( file_offset = 0;  file_offset < 100;  file_offset++)
    {
       if (Name[Stu][file_offset] == 0) break;
       if (Name[Stu][file_offset] == 37) delimiters++;
    }
       
  /*  Skip student if no tests have been taken.  */
  if (delimiters < 3)
    {
       Stu++;
       return 0;
    }

  /*  Start by putting sectionaa template into the buffer.  */
  strcpy (Buf, "");  /*  Clear buffer.  */
  sectionaa();
 
  /*  This loop reads the student name & skips the following password.  */
  for (file_offset = 0; file_offset < 30; file_offset++)
    { 
      if (Name[Stu][file_offset] == 0) break;
      if (Name[Stu][file_offset] == 95) strcat (Buf, " ");
      if (Name[Stu][file_offset] == 37) /*  Skip password  */
        {
          strcat (Buf, " ");
          file_offset = file_offset + 5;
          break;
        }
      if (Name[Stu][file_offset] != 95)
        {
          strncat (Buf, &Name[Stu][file_offset], 1);  /*  Spell name.  */
        }
    }  

  /*  Go get template sectionbb and append it.  */
  sectionbb();

  /*  Next, move print location to right of name.  */
  strcat (Buf," 200");
    
  /*  Append template sectioncc.  */
  sectioncc();

  /*  Next printed line will be at this vertical coordinate.  */
  line_b = 233;

  /*  This loop will read all remaining data in student's file.  */
  while (Name[Stu][file_offset] != 0)
    {
      line_b = line_b + 17;

      /*  Examine each char of student file.  */
      for (; file_offset < 2000; file_offset++)
        {
          /* Look for end of file.  */
          if (Name[Stu][file_offset] == 0) break;

          /*  Look for separator "#".  */
          if (Name[Stu][file_offset] == 35)
            {  
               file_offset++;
               break;
            }

          /*  Change '_' to space.  */
          if (Name[Stu][file_offset] == 95) strcat (Buf, " ");

          /*  Print text.  */
          if ((Name[Stu][file_offset] != 37) & (Name[Stu][file_offset] != 95))
            {
               strncat (Buf, &Name[Stu][file_offset], 1);
            }
        }
     
      strcat (Buf, ") 72 ");
      strcpy(zo, "");
      zo[0] = (char)((line_b / 100) + 48); /*  Line_b starts at 240.  */
      zo[1] = (char)(((line_b % 100) / 10) + 48);
      zo[2] = (char)(((line_b % 100) % 10) + 48);
      strncat (Buf, &zo[0], 3); 
      strcat (Buf, " MS (");
     
                       /*  Append percentage.  */

      /*  Examine each char of student file.  */
      for (; file_offset < 2000; file_offset++)
        {        
          /*  Look for end of file.  */
          if ((Name[Stu][file_offset] == 0) || (Name[Stu][file_offset] == 37))
           {
             break;
           }

          /*  Print 3 characters, changing underlines to spaces.  */
          if (Name[Stu][file_offset] == 95) strcat (Buf, " ");
          else strncat (Buf, &Name[Stu][file_offset], 1);

          file_offset++;  
          if (Name[Stu][file_offset] == 95) strcat (Buf, " ");
          else strncat (Buf, &Name[Stu][file_offset], 1);

          file_offset++; 
          strncat (Buf, &Name[Stu][file_offset], 1);
          strcat (Buf, "%"); /*  Print literal '%'.  */
              
          /*  Append points.  */  
          strcat (Buf, "          "); /* Print under points heading.  */
          file_offset++; /*  Look at tens digit of points.  */

          /*  Change underline to a space.  */
          if (Name[Stu][file_offset] == 95) strcat (Buf, " ");

          /*  Print digit as a character.  */
          else
           {
              strncat (Buf, &Name[Stu][file_offset], 1);
              Name[Stu][file_offset] = Name[Stu][file_offset] - 48;

              /*  Add to tens of total.  */
              total = total + (10 * Name[Stu][file_offset]);
            }
 
          file_offset++; /*  Look at ones digit of points.  */

          /*  Print digit as char.  */
          strncat (Buf, &Name[Stu][file_offset], 1);
          Name[Stu][file_offset] = Name[Stu][file_offset] - 48;

          /*  Add to ones of total.  */
          total = (total + Name[Stu][file_offset]);
              
          /*  Append genre.  */                 
          strcat (Buf, "             "); /*  Move to genre column.  */
          file_offset++; strncat (Buf, &Name[Stu][file_offset], 1);
                
          /*  Append date.  */                    
          strcat (Buf, "        "); /*  Move to Date column.  */
          file_offset++; strncat (Buf, &Name[Stu][file_offset], 1);
          file_offset++; strncat (Buf, &Name[Stu][file_offset], 1);
          file_offset++; strncat (Buf, &Name[Stu][file_offset], 1);
          strcat (Buf, " ");
          file_offset++; strncat (Buf, &Name[Stu][file_offset], 1);
          file_offset++; strncat (Buf, &Name[Stu][file_offset], 1);
          strcat (Buf, ") 325 ");
          strcat (Buf, &zo[0]); /*  Stats position  */
        }
      if ( (int) (Name[Stu][file_offset+1]) == 0) break;
      strcat (Buf, " MS (");  /*  Start next book title.  */    
    }
    
  /*  Append template sectiondd.  */
  sectiondd();

  /*  Print total points.  */
  if (total / 100 == 0) strcat (Buf, " ");
  else
    {
      zo[0] = (char)((total / 100) + 48);
      strncat (Buf, &zo[0], 1);
    }
  if (total % 100 == 0) strcat (Buf, " ");
  else
    {
      zo[0]=(char)(((total % 100) / 10) + 48);
      strncat (Buf, &zo[0], 1);
    }
  zo[0] = (char)((total % 10) + 48);
  strncat (Buf, &zo[0], 1);
  strcat (Buf, ") 269 ");
 
/*  Calculate and add ASCII form of vert print coordinate  */
  line_b = line_b + 25;
  zo[0] = (char)((line_b / 100) + 48);
  strncat (Buf, &zo[0], 1);
  zo[0] = (char)(((line_b % 100) / 10) + 48);
  strncat (Buf, &zo[0], 1);
  zo[0] = (char)(((line_b % 100) % 10) + 48);
  strncat (Buf, &zo[0], 1); 
  strcat (Buf, " ");
  sectionee();

  /*  Upload report in PS format.  */
  fp = fopen ("/usr/local/com/reader/report.ps", "w");
  fputs(Buf, fp);
  fclose(fp);
  
  return 0;
}

/*  This routine formats a video display of student work.  */
int
format_for_video()
{
  int character;
  int file_offset;
  int line_offset = 0;
  int total;

  /*  Prepare to print.  */
  clear();
  attron (COLOR_PAIR(2));
  mvprintw (1, 5, " ");

  /*  Name may be 30 characters long.  */
  for (file_offset = 0; file_offset < 30; file_offset++)
    { 
      character = Name[Stu][file_offset];

      /*  Jump out on endfile.  */
      if (character == 0) break;

      /*  Skip the password.  */
      if (character == 37)
        {
          printw (" ");
          file_offset = file_offset + 5;
          break;
        }
       
      /*  Change _ to a space.  */  
      if (character == 95) printw (" ");

      /*  If it's not a space, print the character.  */
      else printw ("%c", character);
    }

  /*  Append predicate and headings.  */
  attron (COLOR_PAIR(4));
  printw ("  has taken the following tests \n\n\r");
  attron (COLOR_PAIR(1));
  printw ("\t\tTITLE \t\t\t            SCORE  POINTS  GENRE  DATE\n\r");
  total = 0;

  /*  Append tests and scores.  */
  while (Name[Stu][file_offset] != 0)
    {
      /*  Examine each char of the student file.  */
      for (; file_offset < 2000; file_offset++)
        {
          line_offset++;
          character = Name[Stu][file_offset];

          /* Look for the end of the file. */
          if (character == 0) break;

          /*  Change "_" to a space  */
          if (character == 95)
            {
              printw (" ");
            }

          /*  Print alphanumeric characters.  */ 
          if ((character != 35) & (character != 37) & (character != 95))
            {
               printw ("%c", character);
            }

          /*  Look for separator "%".  */
          if (character == 37)
            {
              line_offset = 0;
              printw ("\n "); /*  newline for next book  */
            }

          /*  Look for separator "#".  */
          if (character == 35)
            { 
              for(; 48 - line_offset > 0; line_offset++)
                {
                  printw (" "); /*  Fill title blank to 48 chars  */
                }

                    /*  Append Percentage  */

              printw ("    "); /*  Move to percentage column  */
              file_offset++;
  
              /*  Print three characters, changing any "_" to a space.  */
              if (Name[Stu][file_offset] != 95)
                {
                  printw ("%c", Name[Stu][file_offset]);
                }
              else printw (" ");
              file_offset++;  
              if (Name[Stu][file_offset] != 95)
                {
                  printw ("%c", Name[Stu][file_offset]);
                }
              else printw (" ");
              file_offset++; 
              printw ("%c", Name[Stu][file_offset]);
              printw ("%c", '%');

                          /*  Append points.  */

              /*  Move cursor to points column.  */
              printw ("     ");

              /*  Look at tens digit of points.  */
              file_offset++;

              /*  Print tens and add to tens of total.  */
              if (Name[Stu][file_offset] != 95)
                {
                  printw ("%c", Name[Stu][file_offset]);
                  total = total + (10 * (Name[Stu][file_offset] - 48));
                }
              else printw (" ");

              /*  Look at ones digit of points.  */
              file_offset++;

              /*  Print digit as char.  */
              printw ("%c", Name[Stu][file_offset]);

              /*  Add to ones of total.  */
              total = (total + (Name[Stu][file_offset] - 48));

                           /*  Append Genre  */

              printw ("      ");
              file_offset++;
              printw ("%c", Name[Stu][file_offset]);

                       /*  Append Date  */

              printw ("   "); /*  Move to date column.  */
              file_offset++;
              printw ("%c", Name[Stu][file_offset]);
              file_offset++;
              printw ("%c", Name[Stu][file_offset]);
              file_offset++;
              printw ("%c", Name[Stu][file_offset]);
              printw (" ");
              file_offset++;
              printw ("%c", Name[Stu][file_offset]);
              file_offset++;
              printw ("%c", Name[Stu][file_offset]);
                   
            } 
        }

                       /*  Print total points.  */
 
       attron (COLOR_PAIR(2));
       printw ("\n\t\t\t\t\t      TOTAL POINTS: ");
     
       /*  Print the hundreds.  */
       if ((total / 100) == 0) printw (" ");
       else printw ("%c", ((total/100) + 48));

       /*  Print the tens.  */
       printw ("%c", (((total % 100)/10) + 48));

       /*  Print the ones.  */
       printw ("%c", (((total % 100) % 10) + 48));
     }
   getch();
   return 0;
}

int
print_help()
{
  clear();
  printw (
  "\n\n\n                      HELP CONTENTS\n\n"
  " * Invoking Gnu Eclectic Reader   Getting Started\n"
  " * Class Roster                   Maintaining roster\n"
  " * Test Writing                   A doubtful utility\n"
  " * Booklist                       What's in the library?\n"
  " * Testing                        Did she really read it?\n"     
  " * On Screen Reports              Personal reading record\n"
  " * Printed Report                 Hard copy record\n"
  " * Class Profile                  Class reading record\n"
  " * Security                       Privacy and reliability");
  
  refresh();
  getch();

  clear();
  printw (
  "\n\nPROGRAM INVOCATION\n"
  "\n\n\nStart the program with this command:  greader\n\n"
  "If that fails, try the complete file name: /usr/local/bin/greader\n\n"
  "  \n\nOptions are:\n"
          "  --help     (this file)\n"
          "  --version ");

  refresh();
  getch();

  clear();
  printw (
   "\nCLASS ROSTER\n\n"
   "     There are some areas in this program that should only be\n"
   "accessed by the teacher.  There are other areas where student\n" 
   "privacy may need to be protected.  For this reason, both teachers\n"
   "and students have individual passwords which they may use to\n"
   "exercise their respective privileges.  There is no default for\n"
   "students; you make up individual passwords and enter them when the\n"
   "students are added to the roster.  If security is not a concern\n"
   "for you, just give them all the same 4-digit password.  Make it\n" 
   "something simple, like 'abcd.'\n\n"

   "     In most cases, however, each student will need his own\n" 
   "password.  We suggest that you start by hand-writing a list with\n"
   "the passwords for each student before you start.  Then it will be\n"
   "fairly easy to respond quickly to the program prompts when entering\n"
   "students.  If you make a mistake during this process, it is easy\n"
   "to skip to the Drop Students subroutine and erase the erroneous\n"
   "file.  Then Add Students to re-enter the items correctly.\n\n"

   "     You may examine the list of students to make sure that new\n" 
   "additions or deletions have been properly entered.  Merely push 'C'\n" 
   "in the main menu for a list of students.");

   refresh();
   getch();
   
   clear();
   printw (
   "\nTEST WRITING\n\n"
   "     We are starting with only about 320 titles for which we have\n" 
   "tests. Hopefully this number will rapidly increase, but at the\n"
   "moment it is likely that your students will be reading books for\n" 
   "which you will have to write a test, determine the genre, and\n"
   "calculate the point value.\n\n"  

   "     'Add Book to Library' will allow you to do this, but it might be\n"
   "better not to use it.  Instead you could send an e-mail to the\n"
   "program's maintainer with your title and test questions, and let\n"
   "us create the minifile and insert it into the GER tarball.  That\n"
   "way everybody else will benefit from your work.  Of course, you\n"
   "will need to use the procedure that we outline below to calculate\n"
   "point values.  That way we can be sure that all the book files\n"
   "are designed to the same standards.  But if you are doing this for\n"
   "your own use only, you can use any system or standard you like for\n"
   "assigning points to a book.");

   refresh();
   getch();

   clear();
   printw(
   "\n\nBOOKLIST\n\n"
   "     You can look at the list of book titles by entering B from the\n"
   "main menu. Scroll up and down using the cursor keys.  New book\n"
   "titles and tests that you enter are automatically placed in this\n"
   "list in alphabetical order, using the convention that all upper\n"
   "case letters precede all lower case letters.");

   refresh();
   getch();

   clear();
   printw(
   "\nTESTING\n\n"
   "     A student who has finished reading a book can take the test\n"
   "for that book by entering the Take Test subroutine from the main\n"
   "menu.  After entering his name he will be asked to enter his\n"
   "password.\n\n"  

   "     Once the name is entered, the student uses the cursor-up and\n" 
   "cursor-down keys to select the book to be tested.  There will be a\n"
   "variable number of questions on each test, each in a multiple\n"
   "choice format. Students should consider their answer carefully\n"
   "before pushing the letter of the answer, because selecting the\n"
   "letter of the answer automatically enters it.\n\n"

   "     At the end of the quiz, the machine reports the number of\n"
   "answers that were correct.  Internally, it calculates the percentage\n"
   "correct, and credits the student with points based on the test\n"
   "score and the difficulty level of the book.  These points are added\n"
   "to the cumulative yearly total for later reporting.  The test names,\n"
   "dates, genres, and scores are also added to the permanent record.\n");

   refresh();
   getch();

   clear();
   printw (
   "\nON SCREEEN REPORTS\n\n"
   "     As mentioned in Testing, the program maintains a record of each\n" 
   "test taken.  This record includes the name of the book, the date of\n"
   "the test, the genre of the book, the percentage of correct answers,\n"
   "and the total points the student has earned during the current year\n"
   "(or whatever time period the program has been running).\n\n"

   "     To read this record, enter the Show Student Record by pushing\n" 
   "'D'on the main menu.  After examining the record, push Enter to return\n"
   "to the main menu.");

   refresh();
   getch();

   clear();
   printw(
   "\n\nPRINTED REPORTS\n\n"
   "     Be sure there is a printer attached and configured with CUPS\n"
   "or you will possibly have a crash.\n\n");

   refresh();
   getch();

   clear();
   printw(
   "\n\nCLASS PROFILE\n\n"
   "     This routine will print out a report on a separate page for each\n" 
   "student in your roster, skipping anyone who has not taken a test.\n\n");

   refresh();
   getch();

   clear();
   printw(
   "\nSECURITY\n\n"
   "     As mentioned in Add/Drop Student, security may or may not be a\n" 
   "problem for you.  Minimally, you will want to keep students from\n" 
   "messing with the class roster, the list of books, or other students'\n"
   "records.  That's why you have your own password which you use to\n"
   "access these areas of the program.  There is a default documented\n"
   "in the texinfo file for this program.  Of course, as soon as some\n"
   "kid discovers what it is, you will have to change it.  You can easily\n"
   "do this, however. See the info file for details.\n\n"

   "     To keep the kids from messing with each others files, you first\n"
   "assign each of them a distinct password, as described in Class Roster.\n"
   "Use any convenient means to inform them of their passwords, and warn\n"
   "them not to be sharing them with each other.  There is at present no\n"
   "routine to change a student's password, short of dropping from the\n"
   "roster and then re-entering their name.");

   refresh();
   return 0;
}

int
print_version()
{
  mvprintw (5, 25, "GNU Eclectic Reader Version 1.0.0\n"
  "                          Copyright (C) 2006 John Slayton\n\n\n\n\n"
  "      This program comes with NO WARRANTY, to the extent permitted by law.\n"
  "                You may redistribute copies of Gnu Eclectic Reader\n"
  "                under the terms of the GNU General Public License.\n"
  "      For more information about these matters, see the files named COPYING.");
  refresh();
  return 0;
}

/*  Copy template section aa into Buf.  */
int
sectionaa()
{
strcpy(Buf, ""); /*  Clear buffer.  */
strcat(Buf,"%!PS-Adobe-2.0\n");
strcat(Buf,"%%Creator: AbiWord\n");
strcat(Buf,"%%Orientation: Portrait\n");
strcat(Buf,"%%Pages: 1\n");
strcat(Buf,"%%DocumentPaperSizes: Letter\n");
strcat(Buf,"%%DocumentNeededResources: font BitstreamVeraSerif-Roman\n");
strcat(Buf,"%%EndComments\n");
strcat(Buf,"%%BeginProlog\n");
strcat(Buf,"  /FSF {findfont exch scalefont} bind def\n");
strcat(Buf,"  /MS  {neg moveto show} bind def\n");
strcat(Buf,"  /GS  {glyphshow} bind def\n");
strcat(Buf,"  /MV  {neg moveto} bind def\n");
strcat(Buf,"  /BP  {gsave 72 exch div dup scale} bind def\n");
strcat(Buf,"  /SZ  {/HH exch def /WW exch def} bind def\n");
strcat(Buf,"  /BPP {BP SZ 0 HH translate} bind def\n");
strcat(Buf,"  /BPL {BP SZ 90 rotate} bind def\n");
strcat(Buf,"  /EP  {grestore showpage} bind def\n");
strcat(Buf,"  /ML  {neg moveto neg lineto stroke} bind def\n");
strcat(Buf,"  /LAT {dup length dict copy dup begin\n");
strcat(Buf,"         {1 index /FID ne {def} {pop pop} ifelse} forall\n");
strcat(Buf,"         /Encoding ISOLatin1Encoding 256 array copy def\n");
strcat(Buf,"         Encoding 39 /quotesingle put\n");
strcat(Buf,"         Encoding 45 /hyphen put\n");
strcat(Buf,"         currentdict\n");
strcat(Buf,"         end} bind def\n");
strcat(Buf,"  /EXC {exch definefont pop} bind def\n");
strcat(Buf,"  /CP {closepath} bind def\n");
strcat(Buf,"  /F {fill} bind def\n");
strcat(Buf,"  /LT {neg lineto} bind def\n");
strcat(Buf,"  /MT {neg moveto} bind def\n");
strcat(Buf,"  /F0 {12 /BitstreamVeraSerif-Roman FSF setfont} bind def\n");
strcat(Buf,"  /F1 {14 /BitstreamVeraSerif-Roman FSF setfont} bind def\n");
strcat(Buf,"%%EndProlog\n");
strcat(Buf,"%%BeginSetup\n");
strcat(Buf,"%%IncludeResource: font BitstreamVeraSerif-Roman\n");
strcat(Buf,"%%EndSetup\n");
strcat(Buf,"%%Page: 1 1\n");
strcat(Buf,"%%BeginPageSetup\n");
strcat(Buf,"641 844 72 BPP\n");
strcat(Buf,"%%EndPageSetup\n");
strcat(Buf,"gsave\n");
strcat(Buf,"F1\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"1.00000000 1.00000000 1.00000000 setrgbcolor\n");
strcat(Buf,"0 0 MT\n");
strcat(Buf,"612 0 LT\n");
strcat(Buf,"612 792 LT\n");
strcat(Buf,"0 792 LT\n");
strcat(Buf,"0 0 LT\n");
strcat(Buf,"CP\n");
strcat(Buf,"F\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"1.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"(");
  return 0;
}

/*  Add template section bb to Buf.  */
int
sectionbb()
{
strcat(Buf,") 72 184 MS\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"(has taken the following tests) \n");
  return 0;
}

/*  Add template section cc to Buf.  */
int
sectioncc()
{
strcat(Buf," 184 MS\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"0.00000000 1.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"(                          TITLE                                 SCORE  POINTS   GENRE   DATE) 72 217 MS\n");
strcat(Buf,"(______________________________________________________________________) \n");
strcat(Buf,"72 225 MS \n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf," (");
  return 0;
}

/*  Add template section dd to Buf.  */
int
sectiondd()
{
strcat(Buf," MS  \n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"1.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf," (TOTAL POINTS:    ");
  return 0;
}

/*  Add template section ee to Buf.  */
int
sectionee()
{
strcat(Buf," MS\n");
strcat(Buf,"0.00000000 0.00000000 0.00000000 setrgbcolor\n");
strcat(Buf,"grestore\n");
strcat(Buf,"%%PageTrailer\n");
strcat(Buf,"EP\n");
strcat(Buf,"%%Trailer\n");
strcat(Buf,"%%EOF\n");
  return 0;
}

/*  Select a book from scrolling list.  Return call number in Bknr.  */
int
select_book()
{ 
  int a; 
  Bknr = 0;

  show_books(Bknr);
  
  /*  Scroll up or down or select item.  */
  while (a != 13)
    {
      a = getch();

      /*  Jump out on ESC.  */
      if (a == '\033')
        {
          Panic = 1;
          return 0;
        } 
      /*  Scroll up.  */
      if (a == KEY_DOWN)
        {
          if (Bknr < Library_size - 1)
            {
              Bknr++;
              clear();
              show_books(Bknr);
            }
        }
      /*  Scroll down.  */
      if (a == KEY_UP)
        {
          if (Bknr > 0)
            {
              Bknr--;
             clear(); 
             show_books(Bknr);
            }
        }

    }
  return Bknr;
}

/*  Use scrolling list to select a student from printed roster.  
    Return roster number of selected student in Stu.  */
int
select_student()
{ 
  int a;
  Stu = 0;

  show_roster(Stu);
  
  while (a != 13)
    {
      a = getch();

      /*  Jump out on ESC.  */
      if (a == '\033')
        {
          Panic = 1;
          return 0;
        } 

      if (a == KEY_DOWN)
        {
          if (Stu < Class_size - 1)
            {
              Stu++;
              clear();
              show_roster(Stu);
            }
        }
      if (a == KEY_UP)
        {
          if (Stu > 0)
            {
              Stu--;
             clear(); 
             show_roster(Stu);
            }
        }

    }
  return Stu;
}

/*  Print 20 book titles from library file.  */
int
show_books(int Bknr)
{ 
  int b;
  int n = 0;
  int lib_offset;

  clear();

  /*  Print a title on each of 20 screen lines.  */
  for (lib_offset = 0; (Bknr < Library_size) && (lib_offset < 20); lib_offset++)
    {
      if (Book[Bknr + lib_offset][0] == 0) break;
      mvprintw (lib_offset + 4, 20, " ");
      n = 0;
      while (1 == 1)
        {
          /*  Get a character from file  */
          b = (int)(Book[Bknr + lib_offset][n]);

          /*  Look for end of book title. (delimiter '%')  */ 
          if (b == 37) break;

          /*  Print character, changing any _ to a space.  */
          if (b == 95) printw (" ");
          else   
            {
              printw ("%c", b);
            }     
          n++;
        }
    }

   /*  Prints bullet to left of first title on list.  */
    attron (COLOR_PAIR(2));
    mvprintw (4, 15, ">");
    refresh();
    attron (COLOR_PAIR(4));
    return Stu;
}

/*  Print 20 names from roster.  */
int
show_roster(int Stu)
{ 
  int b;
  int n = 0;
  int file_offset;

  clear();

  /*  Print a name on each of 20 screen lines.  */
  for (file_offset = 0; (Stu < Class_size) && (file_offset < 20); 
  file_offset++)
    {
      if (Name[Stu + file_offset][0] == 0) break;
      mvprintw (file_offset + 4, 20, " ");
      n = 0;
      while (1 == 1)
        {
          /*  Get a character from file  */
          b = (int)(Name[Stu + file_offset][n]);

          /*  Look for end of name. (delimiter '%')  */ 
          if (b == 37) break;

          /*  Print character, changing any _ to a space.  */
          if (b == 95) printw (" ");
          else   
            {
              printw ("%c", b);
            }     
          n++;
        }
    }

   /*  Prints bullet to left of first name on list.  */
    attron (COLOR_PAIR(2));
    mvprintw (4, 15, ">");
    refresh();
    attron (COLOR_PAIR(4));
    return Stu;
}
   
/*  Check password.  */
int
student_clearance()
{
  int a = 0;
  int n;
  char zo[2] = "";
  char code[5];

  /*  Student types in purported password.  */
  clear();
  mvprintw (5, 30, "Enter student ID number");
  mvprintw (8, 30, "");
  curs_set(1);
  strcpy (Buf, "");  /*  Empty the buffer.  */
  for (n = 1; n < 5; n++)
    { 
      a = getch();    
      if (a == '\033')
        {
          Panic = 1;
          return 0;
        }
      if (a == 13) break;
      zo[0] = (char)a;
      strcat(Buf, &zo[0]);
      printw ("*");
      refresh();
    }
  curs_set(0);
  
  /*  Program reads genuine password.  */
  n = 0;
  strcpy (code, "");  /*  Empty the buffer 'code'.  */
  while (Name[Stu][n] != '%') n++;  /*  Move to first separater.  */
  n++;  /*  This is location of first digit of security code.  */
  for (a = 0; a < 4; a++)  /*  Put 4 digits of code into buffer 'code'.  */
    {
      strncat (code, &Name[Stu][n], 1);
      n++;
    }

  /*  Compare entry with password or teacher code and decide validity.  */  
  a = strcmp (code, Buf);
  if (a != 0) Panic = 1;
  a = strcmp (Tc, Buf);
  if (a == 0) Panic = 0;

  clear();
  return 0;
}


int
teacher_clearance()
{
  int a; 
  int b;
  char zo[2] = "";
  clear();
  mvprintw (5, 30, "Enter teacher ID number");
  mvprintw (7, 30, "");
  refresh();
  curs_set(1);

  strcpy (Buf, "");
  for (b = 1; b < 5; b++)
    { 
      a = getch();    
      if (a == '\033')
        {
          Panic = 1;
          return 0;
        }
      if (a == 13) break;
      zo[0] = (char)a;
      strcat(Buf, &zo[0]);
      printw ("*");
      refresh();
    }
  curs_set(0);

  if (((strncmp (Tc, Buf, 4)) != 0) || (strlen (Buf) != 4)) Panic=1;
  clear();
  return 0;
}

/*  This routine presents a selected test to a selected student, creating 
    and maintaining a record of that student's overall test performance.  */
int
testing()
{
  int a;
  int b;  /*  Index used to count the questions  */
  int n;
  int score;
  char dia[8] = ""; 
  char zo[2];

  /*  Get student name and code.  */
  attron (COLOR_PAIR(2));
  mvprintw (2, 10, "FIND YOUR NAME");
  attron (COLOR_PAIR(4));
  select_student(); /*  returns student id number in 'Stu'  */
  if (Panic == 1) return 0;
  student_clearance(); /*  checks student pin number  */
  
  /*  Wrong number, you're outta here!  */
  if (Panic == 1) return 0;
  
  /*  Select a book.  */
  score = 0;
  select_book(); /*  This returns number of book in 'Bknr'.  */
  
  /*  Return to menu if ESC was pushed.  */
  if (Panic == 1) return 0;
  
  /*  Print name of book and add to student record.  */ 
  strcpy (Buf, ""); /*  Clear buffer.  */
  clear(); /*  Clear screen.  */
  attron (COLOR_PAIR(5));
  mvprintw (4, 3, "");
  for (n = 0; n < 60; n++)
    {
      /*  Leave loop when end of title is reached.  */
      if ((Book[Bknr][n] == 0) || (Book[Bknr][n] == 37)) break;

      /*  Change underline to space.  */
      if (Book[Bknr][n] == 95) 
        {
          printw (" "); 
          strcat (Buf, "_");
        }
      else 
        {
          printw ("%c", Book[Bknr][n]); 
          strncat (Buf, &Book[Bknr][n], 1);
        }
    }

  /*  Add book title to Name[stu].  */
  strcat (Name[Stu], Buf);

  /*  Add field delimiter.  */
  strcat (Name[Stu], "#");
       
  /*  Print questions.  */
  b = 0;   
  while (Book[Bknr][n+1] != 126) /*  Use "~" for delimiter.  */
    {
      b++;
      attron (COLOR_PAIR(2));
      mvprintw (10, 3, "");
      for (n = n + 1; n < 1999; n++)
        {
          if (Book[Bknr][n] == 0) break;
          if (Book[Bknr][n] == 37) break; /*  Use "%" for delimiter.  */
          if (Book[Bknr][n] == 35) /*  Use "#" for newline.  */
            {
              attron (COLOR_PAIR(1));
              printw ("\n\n   ");
              n++;
            }
          /*  Change space to underline.  */
          if (Book[Bknr][n] == 95) printw (" ");

          /*  Print alphanumeric text.  */
          else printw ("%c", Book[Bknr][n]);   
        }

      /*  Get student answer between a and d.  */
      refresh();
      a = 0; 
      while ((a < 97) || (a > 100)) a = getch();

      /*  Get past separator 37.  */
      n++;

      /*  Increment score if answer matches key.  */
      if (a == Book[Bknr][n]) score++;

      /*  Get past next separator 37.  */
      n++;
      clear();
    }

  mvprintw (5, 20, "%d", score);
  printw (" right answer");
  if (score != 1) printw ("s");
  mvprintw (22, 3, "Push Enter to Continue");
  refresh();
  
  /*  Change score to percentage.  */
  score = score * 100 / b;    
  if (score == 100) strcat (Name[Stu], "100");
  else
    {
      strcat (Name[Stu], " ");
      zo[0] = (char)( (score/10) + 48);
      strncat (Name[Stu], zo, 1);
      zo[0] = (char)((score % 10) + 48);
      strncat (Name[Stu], zo, 1);
    }
                /*  Add level and points to record.  */

  /*  Get past book file's field delimiter.  */
  n++;
  n++;
 
  /*  Add points.  */  
  if (score > 69) /*  If passing score, add 2 digit point value.  */
    {
      if (Book[Bknr][n] == 48) strcat (Name[Stu], "_");
      else 
        {
          zo[0] = (char) Book[Bknr][n];
          strncat (Name[Stu], zo, 1);
        }
      n++;
      zo[0] = (char) Book[Bknr][n]; 
      strncat (Name[Stu], zo, 1);
    }
  else /*  If not passing, add 2 zeros.  */
    {
      n++;
      strcat (Name[Stu], "00");
    }
  
  /*  Add level.  */
  n++;
  zo[0] = (char) Book[Bknr][n]; 
  strncat (Name[Stu], zo, 1);
  
  /*  Add date to record.  */
  curtime = time (NULL);
  loctime = localtime (&curtime);
  strftime (dia, 25, "%b%d", loctime);
  strcat (Name[Stu], dia);    
  strcat (Name[Stu], "%"); /*  Add endfile.  */
  upload_roster();
  getch();
  clear();
  return 0;
}

/*  Appends standard input text to whatever is in Buf.  */
int
type_anything()
{
  int c;

  /*  Holds string length.  */
  int l = 0;
 
  char zo[2] = "";
  l = strlen(Buf);
      
  curs_set(1);

  while (1 == 1)
    { 
      c = getch();    
      if (c == '\033')
        {
          Panic = 1;
          return 0;
        }

      /*  Get out of while loop on "Enter".  */
      if (c == 13) break;

      if (c == KEY_BACKSPACE)
        {
          /*  Don't back past beginning.  */ 
          if (l == 0) return 0;

          /*  Move endfile back one space.  */
          Buf[l-1] = Buf[l];          
          l--;

          /*  Back up and erase last character  */
          printw ("\b \b");
        }
      else
        {
          printw ("%c", c);

          /*  Change space to underline and add character to Buf.  */
          if (c == 32) c = 95;
          zo[0] = (char) c;
          strcat(Buf, zo);
          l++;  
        }   
    }
  curs_set(0);
  return 0;
}

/*  This routine writes list of books-and-tests to hard drive.  */
int
upload_books()
{
  char buffer1[800000];
  char zo[2];

  /*  Convert Library_size from int to ascii representation & upload.  */
  strcpy (Buf, "");  /*  Clear buffer.  */
  zo[0] = (char) ((Library_size / 100) + 48); /*  Add hundreds digit.  */
  strncat (Buf, &zo[0], 1);
  zo[0] = (char) (((Library_size % 100) / 10) + 48); /*  Add tens digit.  */
  strncat (Buf, &zo[0], 1);
  zo[0] = (char) (((Library_size) % 10) + 48); /*  Add ones digit.  */
  strncat (Buf, &zo[0], 1);

  fp=fopen ("Library_size", "w");
  fputs (Buf, fp);
  fclose (fp);
  strcpy (Buf, ""); /*  Clear buffer.  */          
  strcpy (buffer1, ""); /*  Clear buffer1.  */
           
  /*  Merge all book files into one long file.  */
  for (Bknr = 0; Bknr < Library_size; Bknr ++)
    {
      strcat (buffer1, Book[Bknr]);
      strcat (buffer1, "@");  /*  Use "@" as file separator.  */
    }
  
  /*  Upload the long one.  */
  strcpy (Buf, "bundle");
  fp = fopen (Buf, "w");  /*  Open path to file 'bundle'.  */
  fputs (buffer1, fp);  /*  Store bundled records on hard disk.  */
  fclose (fp);
    
  return 0;
}

/*  Store individual file for each student on hard drive.  */
int
upload_roster()
{
  int a;
  char size[3] = "";
  char zo[2] = "";
  strcpy (Buf, ""); /*  Clear buffer.  */

  /* Convert Class_size from binary to ascii.  */
  zo[0] = (char)((Class_size / 10) + 48);
  strcat (Buf, &zo[0]);
  zo[0] = (char) ((Class_size % 10) + 48);
  strcat (Buf, &zo[0]);

  fp = fopen ("Class_size", "w");
  fputs (Buf, fp);
  fclose(fp);

  for (a = 0; a <= Class_size; a++)
    {
      strcpy (Buf, "");  /*  Clear buffer.  */
      strcpy (Buf, "Name");
      zo[0] = (char)((a / 10) + 48);
      strcat (Buf, &zo[0]);
      zo[0] = (char) ((a % 10) + 48);
      strcat (Buf, &zo[0]);
      fp = fopen (Buf, "w");
      fputs (Name[a], fp);
      fclose (fp);
    }
  return 0;
}
